<?php

namespace MyApp\Controller;

use MyApp\Core\Controller;
use MyApp\Model\GroupsModel as Model;
use MyApp\View\GroupsView as View;
use MyApp\Service\GroupsService as Service;

class GroupsController extends Controller
{
    /**
     * @RouteRegExp = "|^/groups$|"
     */
    public function index()
    {
        $this->htmlResponse((new View())->index((new Service())->getTableData()));
    }

    /**
     * @RouteRegExp = "|^/groups/add$|"
     * @HttpMethod = "GET,POST"
     */
    public function add()
    {
        $service = new Service();

        if (!empty($_POST)) {
            /** @var Model $model */
            $model = $service->addGroup((string) $_POST['name'],
                (int) $_POST['parent']);
            if ($model) {
                $this->redirect('/groups/edit/'.$model->getId());
            }
        }

        $select = $service->sortForSelect($service->getTableData());

        $this->htmlResponse((new View())->add($select));
    }

    /**
     * @RouteRegExp = "|^/groups/delete$|"
     * @HttpMethod = "POST"
     */
    public function delete()
    {
        if (!empty($_POST)) {
            (new Service())->deleteGroup((int) $_POST['id']);
        }
    }

    /**
     * @RouteRegExp = "|^/groups/edit/(?<id>\d+)$|"
     * @HttpMethod = "GET,POST"
     *
     * @param $id
     *
     * @throws \Exception
     */
    public function edit($id)
    {
        $service = new Service();

        $model = !empty($_POST) ? $service->updateGroup((int) $id,
            (string) $_POST['name'],
            (int) $_POST['parent']) : $service->getGroup((int) $id);

        if ($model) {
            $select = $service->sortForSelect($service->getTableData());

            $this->htmlResponse((new View())->edit($model, $select));
        } else {
            $this->redirect('/groups');
        }
    }
}
