<?php

namespace MyApp\Service;

use MyApp\Model\GroupsModel;
use MyApp\Core\Model;
use Exception;

class GroupsService
{
    /**
     * @return GroupsModel[]|null
     *
     * @throws Exception
     */
    public function getTableData(): ?array
    {
        $models = GroupsModel::find(null, null, '1 = 1');

        return $models;
    }

    public function getGroup(int $id): ?GroupsModel
    {
        /** @var GroupsModel[] $model */
        $model = GroupsModel::find([$id], null, null, [], '', '1');

        if (!empty($model)) {
            return $model[0];
        }

        return null;
    }

    public function sortForSelect($models, $parent_id = null, $level = 1)
    {
        $out = [];
        foreach ($models as $item) {
            /** GroupsModel $item */
            if (
                $item->getParent() === $parent_id) {
                $item->{'select-level'} = str_repeat('-', $level);
                $out[] = $item;
                $out = array_merge($out,
                    $this->sortForSelect($models, $item->getId(), $level + 1));
            }
        }

        return $out;
    }

    public function updateGroup(int $id, string $name, int $parent): ?Model
    {
        if ($model = $this->getGroup($id)) {
            $model->setName($name)
                  ->setParent($parent ? $parent : null);
            Model::save([$model]);
        }

        return $model;
    }

    public function deleteGroup(int $id): void
    {
        if ($model = $this->getGroup($id)) {
            $arr = [$model];
            Model::delete($arr);
        }
    }

    public function addGroup(string $name, int $parent): ?Model
    {
        $model = (new GroupsModel())->setName($name)
                                    ->setParent($parent ? $parent : null);
        Model::save([$model]);

        return $model;
    }
}
